package GitLab::Members;

use utf8;
use strict;
use warnings;
use vars qw($VERSION);

use GitLab::API;
use Log::Any        qw($log);

our $VERSION = v8.17.2;

my $requests = {
    group_members   => {
        method      => "GET",
        path        => "/groups/<gid>/members",
        paginated   => 1,
        optional    => [qw(
            query
        )],
    },

    group_get_member => {
        method      => "GET",
        path        => "/groups/<gid>/members/<user_id>",
    },

    group_add_member => {
        method      => "POST",
        path        => "/groups/<gid>/members",
        required    => [qw(
            user_id
            access_level
        )],
        optional    => [qw(
            expires_at
        )],
    },

    group_update_member => {
        method      => "PUT",
        path        => "/groups/<gid>/members/<user_id>",
        required    => [qw(
            access_level
        )],
        optional    => [qw(
            expires_at
        )],
    },

    group_delete_member => {
        method      => "DELETE",
        path        => "/groups/<gid>/members/<user_id>",
    },

    project_members => {
        method      => "GET",
        path        => "/projects/<id>/members",
        paginated   => 1,
        optional    => [qw(
            query
        )],
    },

    project_get_member => {
        method      => "GET",
        path        => "/projects/<id>/members/<user_id>",
    },

    project_add_member => {
        method      => "POST",
        path        => "/projects/<id>/members",
        required    => [qw(
            user_id
            access_level
        )],
        optional    => [qw(
            expires_at
        )],
    },

    project_update_member => {
        method      => "PUT",
        path        => "/projects/<id>/members/<user_id>",
        required    => [qw(
            access_level
        )],
        optional    => [qw(
            expires_at
        )],
    },

    project_delete_member => {
        method      => "DELETE",
        path        => "/projects/<id>/members/<user_id>",
    },
};

sub import {
    $log->debug("initializing " . __PACKAGE__);
    while (my ($name, $tmpl) = each(%$requests)) {
        $tmpl->{name} = $name unless exists $tmpl->{name};
        GitLab::API->register($tmpl);
    }
}

1;

__END__

=head1 NAME

GitLab::Members - implements group and project members API calls

See L<GitLab API -- Group and Project Members|https://docs.gitlab.com/ce/api/members.html> for details and
response formats.

=head1 VERSION

Implements API calls for GitLab CE C<v8.17.2>.

=head1 DESCRIPTION

=head2 Notation

Please see the documentation for the L<GitLab::Users> module.

Note that not all optional arguments are listed.
Please refer to the official documentation for the full list.

=head2 Group Members

=over

=item L<group_members()|https://docs.gitlab.com/ce/api/members.html#list-all-members-of-a-group-or-project>

    $members = $gitlab->group_members( :gid );

Returns a list of members in the group with the specified C<gid>.

=item L<group_get_member()|https://docs.gitlab.com/ce/api/members.html#get-a-member-of-a-group-or-project>

    $gitlab->group_get_member( :gid, :user_id )

Returns the information about a member of the group.
Can also be used to check whether the given user is a member of the group if used with C<< -immortal => 1 >>:

    if (defined $gitlab->group_get_member(gid => $gid, $user_id => $user_id, -immortal => 1)) {
        # is a member
    } else {
        # not a member
    }

=item L<group_add_member()|https://docs.gitlab.com/ce/api/members.html#add-a-member-to-a-group-or-project>

    $gitlab->group_add_member( :gid, :user_id, :access_level, [:expires_at] );

Adds a user with C<user_id> to the group with the given C<gid> with the specified C<access_level>.
See L<GitLab::API> module for possible access level values.
The optional argument C<expires_at> expects format C<YYYY-MM-DD>.

=item L<group_update_member()|https://docs.gitlab.com/ce/api/members.html#edit-a-member-of-a-group-or-project>

    $gitlab->group_update_member( :gid, :user_id, :access_level, [:expires_at] );

For user with the given C<user_id> the function changes his C<access_level> in the group with the specified C<gid>.
It can also remove the C<expires_at> property if set to C<undef>.

=item L<group_delete_member()|https://docs.gitlab.com/ce/api/members.html#remove-a-member-from-a-group-or-project>

    $gitlab->group_delete_member( :gid, :user_id );

Removes the user with C<user_id> from the group with the specified C<gid>.

=head2 Project Members

=over

=item L<project_members()|https://docs.gitlab.com/ce/api/members.html#list-all-members-of-a-group-or-project>

    $members = $gitlab->project_members( :gid );

Returns a list of members in the project with the specified C<id>.

=item L<project_get_member()|https://docs.gitlab.com/ce/api/members.html#get-a-member-of-a-group-or-project>

    $gitlab->project_get_member( :gid, :user_id )

Returns the information about a member of the project.
Can also be used to check whether the given user is a member of the project if used with C<< -immortal => 1 >>:

    if (defined $gitlab->project_get_member(id => $id, $user_id => $user_id, -immortal => 1)) {
        # is a member
    } else {
        # not a member
    }

=item L<project_add_member()|https://docs.gitlab.com/ce/api/members.html#add-a-member-to-a-group-or-project>

    $gitlab->project_add_member( :gid, :user_id, :access_level, [:expires_at] );

Adds a user with C<user_id> to the project with the given C<id> with the specified C<access_level>.
See L<GitLab::API> module for possible access level values.
The optional argument C<expires_at> expects format C<YYYY-MM-DD>.

=item L<project_update_member()|https://docs.gitlab.com/ce/api/members.html#edit-a-member-of-a-group-or-project>

    $gitlab->project_update_member( :gid, :user_id, :access_level, [:expires_at] );

For user with the given C<user_id> the function changes his C<access_level> in the group with the specified C<gid>.
It can also remove the C<expires_at> property if set to C<undef>.

=item L<group_delete_member()|https://docs.gitlab.com/ce/api/members.html#remove-a-member-from-a-group-or-project>

    $gitlab->group_delete_member( :gid, :user_id );

Removes the user with C<user_id> from the group with the specified C<gid>.

=back

=head1 AUTHOR

Roman Lacko <L<xlacko1@fi.muni.cz>>

=head1 SEE ALSO

=over

=item L<GitLab>

Wrapper around L<GitLab::API> and other C<GitLab::*> modules.

=back
