package GitLab::Users;

use utf8;
use strict;
use warnings;
use vars qw($VERSION);

use GitLab::API;
use Log::Any        qw($log);

our $VERSION = 8.17.2;

my $requests = {
    users => {
        method      => "GET",
        path        => "/users",
        query       => [qw(
            username
            search
            blocked
            active
        )],
        paginated   => 1,
    },

    user_by_id => {
        method      => "GET",
        path        => "/users/<uid>",
    },

    user_create     => {
        method      => "POST",
        path        => "/users",
        required    => [qw(
            email
            username
            name
        )],
        optional    => [qw(
            password
            reset_password
            skype
            linkedin
            twitter
            website_url
            organization
            projects_limit
            extern_uid
            provider
            bio
            location
            admin
            can_create_group
            confirm
            external
        )],
    },

    user_update     => {
        method      => "PUT",
        path        => "/users/<uid>",
        # update can cause Conflict, but API always returns 404
        ret         => {
            404 => "Not found or Conflict"
        },
        optional    => [qw(
            email
            username
            name
            password
            skype
            linkedin
            twitter
            website_url
            organization
            projects_limit
            extern_uid
            provider
            bio
            location
            admin
            can_create_group
            confirm
            external
        )],
    },

    user_delete     => {
        method      => "DELETE",
        path        => "/users/<uid>",
    },

    user => {
        method      => "GET",
        path        => "/user",
    },

    self_ssh_keys => {
        method      => "GET",
        path        => "/user/keys",
        paginated   => 1,
    },

    self_get_ssh_key => {
        method      => "GET",
        path        => "/user/keys/<keyid>",
    },

    user_get_ssh_keys => {
        method      => "GET",
        path        => "/users/<uid>/keys",
        paginated   => 1,
    },

    self_add_ssh_key => {
        method      => "POST",
        path        => "/user/keys",
        required    => [ qw!title key! ],
    },

    user_add_ssh_key => {
        method      => "POST",
        path        => "/users/<uid>/keys",
        required    => [ qw!title key! ],
    },

    self_delete_ssh_key => {
        method      => "DELETE",
        path        => "/user/keys/<keyid>",
    },

    user_delete_ssh_key => {
        method      => "DELETE",
        path        => "/users/<uid>/keys/<keyid>",
    },

    self_get_emails => {
        method      => "GET",
        path        => "/user/emails",
        paginated   => 1,
    },

    user_get_emails => {
        method      => "GET",
        path        => "/users/<uid>/emails",
        paginated   => 1,
    },

    self_get_email => {
        method      => "GET",
        path        => "/user/emails/<eid>",
    },

    # not officially supported, may not work :(
    # user_get_email => {
    #     method      => "GET",
    #     path        => "/users/<uid>/emails/<eid>",
    # },

    self_add_email => {
        method      => "POST",
        path        => "/user/emails",
        required    => [ qw!email! ],
    },

    user_add_email => {
        method      => "POST",
        path        => "/users/<uid>/emails",
        required    => [ qw!email! ],
    },

    self_delete_email => {
        method      => "DELETE",
        path        => "/user/emails/<eid>",
    },

    user_delete_email => {
        method      => "DELETE",
        path        => "/users/<uid>/emails/<eid>",
    },

    user_block      => {
        method      => "PUT",
        path        => "/users/<uid>/block",
    },

    user_unblock    => {
        method      => "PUT",
        path        => "/users/<uid>/unblock",
    },

    user_events     => {
        method      => "GET",
        path        => "/users/<uid>/events",
        paginated   => 1,
    },
};

sub import {
    $log->debug("initializing " . __PACKAGE__);
    while (my ($name, $tmpl) = each(%$requests)) {
        $tmpl->{name} = $name unless exists $tmpl->{name};
        GitLab::API->register($tmpl);
    }
}

1;

__END__

=head1 NAME

GitLab::Users - implements user API calls

See L<GitLab API -- Users|http://doc.gitlab.com/ce/api/users.html> for details and
response formats.

=head1 VERSION

Implements API calls for GitLab CE C<v8.17.2>.

=head1 SYNOPSIS

    use GitLab::API;
    use GitLab::Users;
    # all calls from GitLab::Users are "injected" into the GitLab::API as methods

    my $api = GitLab::API->new();

    my $john = $api->users(username => "john_doe");

    $api->user_create(
        email       => "jane34@somewhere.com",
        password    => "s3cre1",
        username    => "jane.doe",
        name        => "Jane Doe"
    );

=head1 DESCRIPTION

=head2 Notation

All methods take a hash of arguments as specified in the documentation for
L<GitLab::API/exec_request>, but to simplify the documentation,
we will use the following notation:

    users( :username, :search )

which means that the C<users> method takes a hash with two keys, C<username>
and C<search>, so it can be called as

    $gitlab->users(search => "john_doe");

Optional arguments are denoted as C< [:username] >.
Note that not all optional arguments are listed.
Please refer to the official documentation for the full list.

=head2 User CRUD operations

=over

=item L<users()|https://docs.gitlab.com/ce/api/users.html#list-users>

    $users = $gitlab->users( [:username], [:search] );

This method returns all users. The optional arguments can be used
to filter out the results:

    username        search for a user by his username (exact match)
    search          search for users with username or email (substring match)

Note that the returned value is still an array even when C<< users(username => $username) >>
always returns at most one result.

=item L<user_by_id()|https://docs.gitlab.com/ce/api/users.html#single-user>

    $user = $gitlab->user_by_id( :uid );

Returns a user with the given C<uid>.

=item L<user_create()|https://docs.gitlab.com/ce/api/users.html#user-creation>

    $user = $gitlab->user_create( :email, :password, :username, :name );

Creates a new user and if succeeds, returns the generated account information.

    email               primary email, must not exist in GitLab,
    password            user password,
    username            login, must not exist in GitLab,
    name                display name

The function can take many optional arguments. Please see the API documentation for
the full list. Most interesting arguments are:

    admin               create an admin account (bool)
    can_create_group    the user will be able to create group (bool)
    external            create an external account (bool)
    projects_limit      maximum number of projects the user can own

=item L<user_update()|https://docs.gitlab.com/ce/api/users.html#user-modification>

    $user = $gitlab->user_update( :uid );

Updates the user with the given C<uid>. It takes the same arguments as C<user_create>
except they are all optional. Note that though it is possible to change user's primary e-mail
and username, it is probably not a very sane thing to do.

Also, from documentation:

    Note, at the moment this method does only return a 404 error, even in cases where
    a 409 (Conflict) would be more appropriate, e.g. when renaming the email address to some existing one.

=item L<user_delete()|https://docs.gitlab.com/ce/api/users.html#user-deletion>

    $gitlab->user_delete( :uid );

Deletes the user with the given C<uid>.

Note that

    calling this function for a non-existent user id still returns a status code 200 OK.
    The JSON response differs if the user was actually deleted or not.

(from the GitLab API documentation).

=item L<user()|https://docs.gitlab.com/ce/api/users.html#user>

    $user = $gitlab->user();

Returns the user whose authentication key is used for the request
(or impersonated user if C<sudo> is in effect).

Behaves the same as the L<GitLab::API/whoami> method.

=back

=head2 SSH keys

=over

=item L<self_ssh_keys()|https://docs.gitlab.com/ce/api/users.html#list-ssh-keys>

    $keys = $gitlab->self_ssh_keys();

Returns a list of SSH keys for the user that makes the request.

=item L<self_get_ssh_key()|https://docs.gitlab.com/ce/api/users.html#list-ssh-keys-for-user>

    $key = $gitlab->self_get_ssh_key( :keyid );

For the user that makes the request, the function returns the SSH key with the given C<keyid>.

=item L<user_get_ssh_keys()|https://docs.gitlab.com/ce/api/users.html#single-ssh-key>

    $keys = $gitlab->user_get_ssh_keys( :uid );

Returns a list of SSH keys for the user with the given C<uid>.

=item L<self_add_ssh_key()|https://docs.gitlab.com/ce/api/users.html#add-ssh-key>

    $key = $gitlab->self_add_ssh_key( :title, :key );

Adds a new key for the user that makes the request. Required arguments are

    title               the title of the key
    key                 public SSH key

=item L<user_add_ssh_key()|https://docs.gitlab.com/ce/api/users.html#add-ssh-key-for-user>

    $key = $gitlab->user_add_ssh_key( :uid, :title, :key );

Similar to C<self_add_ssh_key>, but targets the user with the given C<uid>.

=item L<self_delete_ssh_key()|https://docs.gitlab.com/ce/api/users.html#delete-ssh-key-for-current-user>

    $gitlab->self_delete_ssh_key( :keyid )

Removes the SSH key with the given C<keyid> for the user that makes the request.

=item L<user_delete_ssh_key()|https://docs.gitlab.com/ce/api/users.html#delete-ssh-key-for-given-user>

    $gitlab->user_delete_ssh_key( :uid, :keyid );

Removes the SSH key with the C<keyid> for the user with the given C<uid>.

=back

=head2 E-mails

=over

=item L<self_get_emails()|https://docs.gitlab.com/ce/api/users.html#list-emails>

    $emails = $gitlab->self_get_emails();

Returns a list of B<secondary> e-mails for the user that makes the request.

Note that primary e-mail can be obtained from the result of C<whoami>.

=item L<user_get_emails()|https://docs.gitlab.com/ce/api/users.html#list-emails-for-user>

    $emails = $gitlab->user_get_emails( :uid );

Returns a list of B<secondary> e-mails for the user with the C<uid>.

Note that primary e-mail can be obtained from the result of C<user_by_id>.

=item L<self_get_email()|https://docs.gitlab.com/ce/api/users.html#single-email>

    $email = $gitlab->self_get_email( :eid );

Returns the e-mail with the given C<eid> of the user that makes the request.

=item L<self_add_email()|https://docs.gitlab.com/ce/api/users.html#add-email>

    $email = $gitlab->self_add_email( :email );

Adds a new C<email> for the user that makes the request. The e-mail must
not exist in GitLab.

=item L<user_add_email()|https://docs.gitlab.com/ce/api/users.html#add-email-for-user>

    $email = $gitlab->user_add_email( :uid, :email );

Adds a new C<email> for the user with the given C<uid>. The e-mail must
not exist in GitLab.

=item L<self_delete_email()|https://docs.gitlab.com/ce/api/users.html#delete-email-for-current-user>

    $gitlab->self_delete_email( :eid );

Deletes the e-mail with C<eid> of the user that makes the request.

=item L<user_delete_email()|https://docs.gitlab.com/ce/api/users.html#delete-email-for-given-user>

    $gitlab->user_delete_email( :uid, :eid );

Removes the e-mail with C<eid> of the user with the given C<uid>.

=back

=head2 Account blocking

=over

=item L<user_block()|https://docs.gitlab.com/ce/api/users.html#block-user>

    $gitlab->user_block( :uid );

Blocks the user with C<uid> in the GitLab.

=item L<user_unblock()|https://docs.gitlab.com/ce/api/users.html#unblock-user>

    $gitlab->user_unblock( :uid );

Unblocks the user with C<uid> in the GitLab.
Note that LDAP blocked users cannot be unblocked by this function.
In that case the API would return C<403 Forbidden>.

=back

=head2 Miscellaneous

=over

=item L<user_events()|https://docs.gitlab.com/ce/api/users.html#get-user-contribution-events>

    $gitlab->user_events( :uid );

Get the contribution events for the specified user, sorted from newest to oldest.

=back

=head1 AUTHOR

Roman Lacko <L<xlacko1@fi.muni.cz>>

=head1 SEE ALSO

=over

=item L<GitLab>

Wrapper around L<GitLab::API> and other C<GitLab::*> modules.

=back
